<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @since      1.0.0
 *
 * @package    Wp2leads
 * @subpackage Wp2leads/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Wp2leads
 * @subpackage Wp2leads/includes
 */
class Wp2leads {

    public static $wp2l_license_check = false;

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Wp2leads_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'WP2LEADS_VERSION' ) ) {
			$this->version = WP2LEADS_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'wp2leads';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Wp2leads_Loader. Orchestrates the hooks of the plugin.
	 * - Wp2leads_i18n. Defines internationalization functionality.
	 * - Wp2leads_Admin. Defines all hooks for the admin area.
	 * - Wp2leads_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp2leads-license.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp2leads-rest-api.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp2leads-klicktipp-connector.php';

        /**
         * Library for setting up Background processes
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp2leads-background.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp2leads-background-maptoapi.php';

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp2leads-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp2leads-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-wp2leads-admin-ajax.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-wp2leads-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-wp2leads-public.php';

        /**
         * The class responsible for defining all actions for cron.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wp2leads-admin-cron.php';

		$this->loader = new Wp2leads_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Wp2leads_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Wp2leads_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {
		$plugin_admin = new Wp2leads_Admin($this->get_plugin_name(), $this->get_version());
		$cron_manager = new Wp2LeadsCron;
		$ajax_handler = new Wp2leads_Admin_Ajax;

        $cron_manager->setScheduleHook();

		$this->loader->add_action('plugins_loaded', $plugin_admin, 'update_db_check');
		$this->loader->add_action('admin_init', $plugin_admin, 'set_license');


        $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_global_scripts' );

		if(isset($_GET['page']) && $_GET['page'] == 'wp2l-admin') {
            $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
            $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
        }
		$this->loader->add_action('admin_menu', $plugin_admin, 'add_menu');

		// Add admin bar menu for displaying BG Processes
        //$this->loader->add_action('admin_bar_menu', $plugin_admin, 'admin_bar_menu', 100);
		$this->loader->add_action('admin_notices', $plugin_admin, 'add_admin_notices');
		$this->loader->add_action('admin_notices', $plugin_admin, 'add_admin_map_to_api_bg_notices');

        $this->loader->add_filter('plugin_action_links_' . plugin_basename(dirname(dirname(__FILE__))) . '/wp2leads.php', $plugin_admin, 'add_plugin_screen_link');
        $this->loader->add_filter('upload_mimes', $plugin_admin, 'allowed_mimes');

        // TODO - Old prepare data script
        $this->loader->add_action('wp_ajax_wp2l_prepare_data_for_klicktipp', $ajax_handler, 'ajax_prepare_data_for_klicktipp');
        // TODO: Test Prepare data for KT
        $this->loader->add_action('wp_ajax_wp2l_prepare_data_for_klicktipp_bg', $ajax_handler, 'prepare_data_for_klicktipp_bg');
        $this->loader->add_action('wp_ajax_wp2l_prepare_data_for_klicktipp_for_cron', $ajax_handler, 'prepare_data_for_klicktipp_for_cron');

        $this->loader->add_action('wp_ajax_wp2l_get_transfer_modal_data_info', $ajax_handler, 'get_transfer_modal_data_info');


        // TODO - New Transfer data to KT
        $this->loader->add_action('wp_ajax_wp2l_transfer_all_data_to_klicktip_bg', $ajax_handler, 'transfer_all_data_to_klicktip_bg');

        // TODO - New Transfer current user data to KT
        $this->loader->add_action('wp_ajax_wp2l_transfer_current_data_to_klicktip_bg', $ajax_handler, 'transfer_current_to_klicktipp');
        $this->loader->add_action('wp_ajax_wp2l_update_existed_tag_fieldset_list', $ajax_handler, 'update_existed_tag_fieldset_list');
        $this->loader->add_action('wp_ajax_wp2l_terminate_map_to_api', $ajax_handler, 'terminate_map_to_api');
        $this->loader->add_action('wp_ajax_wp2l_refresh_all_map_to_api', $ajax_handler, 'refresh_all_map_to_api');
        $this->loader->add_action('wp_ajax_wp2l_get_map_to_api_statistics', $ajax_handler, 'get_map_to_api_statistics');
        $this->loader->add_action('wp_ajax_wp2l_terminate_all_map_to_api', $ajax_handler, 'terminate_all_map_to_api');

        // TODO - Old Transfer data to KT
        $this->loader->add_action('wp_ajax_wp2l_transfer_to_klicktipp', $ajax_handler, 'ajax_transfer_to_klicktipp');


        $this->loader->add_action('wp_ajax_wp2l_transfer_to_klicktipp_bg', $ajax_handler, 'transfer_to_klicktipp_bg');
        $this->loader->add_action('wp_ajax_get_another_page_data', $ajax_handler, 'ajax_get_another_page_data');
        $this->loader->add_action('wp_ajax_wp2l_get_available_options_data', $ajax_handler, 'get_available_options_data');
        $this->loader->add_action('wp_ajax_wp2l_get_subscriber_tags_from_klicktipp', $ajax_handler, 'ajax_get_subscriber_tags_from_klicktipp');
        $this->loader->add_action('wp_ajax_wp2l_load_possible_tags_cloud', $ajax_handler, 'load_possible_tags_cloud');
        $this->loader->add_action('wp_ajax_wp2l_save_new_map', $ajax_handler, 'ajax_save_new_map');
        $this->loader->add_action('wp_ajax_wp2l_map_delete', $ajax_handler, 'ajax_map_delete');
        $this->loader->add_action('wp_ajax_wp2l_fetch_map_query_results', $ajax_handler, 'ajax_fetch_map_query_results');
        $this->loader->add_action('wp_ajax_wp2l_get_map_query_results_by_map_id', $ajax_handler, 'get_map_query_results_by_map_id');
        $this->loader->add_action('wp_ajax_wp2l_get_map_query_results_by_map_id_limited', $ajax_handler, 'get_map_query_results_by_map_id_limited');
        $this->loader->add_action('wp_ajax_wp2l_get_mapping', $ajax_handler, 'ajax_get_mapping');
        $this->loader->add_action('wp_ajax_wp2l_maps_actions_run', $ajax_handler, 'maps_actions_run');
        $this->loader->add_action('wp_ajax_wp2l_maps_import_from_remote', $ajax_handler, 'import_from_remote');
        $this->loader->add_action('wp_ajax_wp2l_get_map_rows_count', $ajax_handler, 'get_map_rows_count');
        $this->loader->add_action('wp_ajax_wp2l_get_map_query_results_limit', $ajax_handler, 'get_map_query_results_limit');

        $this->loader->add_action('wp_ajax_wp2l_export_maps', $ajax_handler, 'export_maps');
        $this->loader->add_action('wp_ajax_wp2l_import_maps', $ajax_handler, 'import_maps');
        $this->loader->add_action('wp_ajax_wp2l_import_pending_maps', $ajax_handler, 'import_pending_maps');

        $this->loader->add_action('wp_ajax_wp2l_license_activation', $ajax_handler, 'ajax_license_activation');
        $this->loader->add_action('wp_ajax_wp2l_modal_license_activation', $ajax_handler, 'ajax_modal_license_activation');
        $this->loader->add_action('wp_ajax_wp2l_license_deactivation', $ajax_handler, 'ajax_license_deactivation');
        $this->loader->add_action('wp_ajax_wp2l_license_removing', $ajax_handler, 'ajax_license_removing');
        $this->loader->add_action('wp_ajax_wp2l_complete_activation', $ajax_handler, 'ajax_complete_activation');

        $this->loader->add_action('wp_ajax_wp2l_set_transient', $ajax_handler, 'set_transient');
        $this->loader->add_action('wp_ajax_wp2l_get_transient', $ajax_handler, 'get_transient');
        $this->loader->add_action('wp_ajax_wp2l_delete_transient', $ajax_handler, 'delete_transient');

        $this->loader->add_action('wp_ajax_wp2l_save_results_to_transient', $ajax_handler, 'save_results_to_transient');

        // Multisearch
        $this->loader->add_action('wp_ajax_wp2l_global_multisearch_results', $ajax_handler, 'ajax_get_global_multisearch_results');
        $this->loader->add_action('wp_ajax_wp2l_global_table_search_results', $ajax_handler, 'ajax_get_global_table_search_results');
        $this->loader->add_action('wp_ajax_wp2l_single_multisearch_table', $ajax_handler, 'ajax_get_single_multisearch_table');

        $this->loader->add_action('wp_ajax_wp2l_fetch_tables', $plugin_admin, 'ajax_fetch_tables');
        $this->loader->add_action('wp_ajax_wp2l_fetch_all_columns_for_map', $plugin_admin, 'ajax_fetch_all_columns_for_map');
        $this->loader->add_action('wp_ajax_wp2l_fetch_column_options', $plugin_admin, 'ajax_fetch_column_options');
        $this->loader->add_action('wp_ajax_wp2l_debug_fetch_query_for_map', $plugin_admin, 'ajax_debug_fetch_query_for_map');

        $this->loader->add_action('wp_ajax_wp2l_detach_from_tag', $plugin_admin, 'ajax_detach_from_tag');
        $this->loader->add_action('wp_ajax_wp2l_transfer_to_klicktipp_last', $plugin_admin, 'ajax_transfer_to_klicktipp_last');
        $this->loader->add_action('wp_ajax_wp2l_klicktipp_get_speed', $plugin_admin, 'klicktipp_getSpeed');
        $this->loader->add_action('wp_ajax_wp2l_settings_klicktipp', $plugin_admin, 'ajax_settigs_klicktipp');
        $this->loader->add_action('wp_ajax_add_new_klick_tip_tag', $plugin_admin, 'add_new_klick_tip_tag');
        $this->loader->add_action('wp_ajax_remove_klick_tip_tag', $plugin_admin, 'remove_klick_tip_tag');

        $this->loader->add_action('wp_ajax_wp2l_save_cron_settings', $cron_manager, 'save_cron_status');

        $this->loader->add_action('wp2lead_after_license_server_response', $plugin_admin, 'delete_server_status_messages', 20, 2);
        $this->loader->add_action('wp2lead_after_license_server_response', $plugin_admin, 'set_server_status_messages', 20, 2);

        // Extend rest api
        $this->loader->add_action('rest_api_init', $plugin_admin, 'rest_api_init');

        // TODO - This is only for DEV purposes need to be removed
        $this->loader->add_action('wp_ajax_wp2l_set_fake_license_level', $plugin_admin, 'set_fake_license_level');

//        add_shortcode('wp2l-klicktipp-speed', ['Wp2leads_Admin', 'klicktipp_getSpeed']);
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Wp2leads_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Wp2leads_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
